@@ -35,8 +35,9 @@ class MyTextWrapper(textwrap.TextWrapper):
 
         # Override wrapping regex, preserve '/' before linebreak
         self.wordsep_re = re.compile(
-            u'(/|'                                    # a forward slash  
+            u'('
             u'[ \t\n\r\f\v]+|'                        # any ASCII whitespace
+            u'[^\\s\\w]*\\w+/(?=[A-Za-z]\\w*)|'               # forward-slash separated words
             u'[^\\s\\w]*\\w+[^0-9\\W]-(?=\\w+[^0-9\\W])|'   # hyphenated words
             u'''(?<=[\\w\\!"'\\&\\.\\,\\?])-{2,}(?=\\w))'''    # em-dash
             u'(?![\u2060|\u200B])')                   # UNLESS &wj; or &zwbs; or &nbsp;
@@ -53,10 +54,10 @@ class MyTextWrapper(textwrap.TextWrapper):
         # Exceptions which should not be treated like end-of-sentence
         self.not_sentence_end_re = re.compile(
             # start of string or non-alpha character
-            r'(^|[^A-Za-z])'
+            r'(^|\s)'
             r'('
             # Single uppercase letter, dot, enclosing parentheses or quotes
-            r'[A-Za-z]\.[\]\)\'"]*'
+            r'\.[\]\)\'"]*'
             # Tla with leading uppercase, and special cases
             # (Note: v1 spelled out Fig, Tbl, Mrs, Drs, Rep, Sen, Gov, Rev, Gen, Col, Maj and Cap,
             #  but those are redundant with the Tla regex.)
@@ -162,7 +163,7 @@ def justify_inline(left_str, center_str, right_str, width=72):
         Raises a warning if the combined length of the three strings is
         greater than the width, and trims the longest string
     """
-    strings = [left_str.strip(), center_str.strip(), right_str.strip()]
+    strings = [left_str.rstrip(), center_str.strip(), right_str.strip()]
     sumwidth = sum( [len(s) for s in strings] )
     if sumwidth > width:
         # Trim longest string
