@@ -4,6 +4,7 @@
 
 import codecs
 import datetime
+import textwrap
 import lxml
 import xml2rfc.log
 import xml2rfc.utils
@@ -337,6 +338,18 @@ class BaseRfcWriter:
         if last and anchor:
             last.anchor = anchor
 
+    def _add_iref_to_index(self, element):
+        item = element.attrib.get('item', None)
+        if item:
+            subitem = element.attrib.get('subitem', None)
+            self._make_iref(item, subitem)
+            # Store the buffer position for pagination data later
+            pos = len(self.buf)
+            if not self.indexmode:
+                if pos not in self.iref_marks:
+                    self.iref_marks[pos] = []
+                self.iref_marks[pos].append((item, subitem))
+
     def _indexParagraph(self, counter, p_counter, anchor=None, toc=False):
         counter = str(counter)  # This is the section counter
         p_counter = str(p_counter)  # This is the paragraph counter
@@ -385,7 +398,7 @@ class BaseRfcWriter:
         counter = str(counter)
         autoName = 'Figure' + self.nbws_cond + counter
         autoAnchor = 'rfc.figure.' + counter
-        item = _RfcItem(autoName, autoAnchor, title=title, anchor=anchor, \
+        item = _RfcItem(autoName, autoAnchor, counter=counter, title=title, anchor=anchor, \
                        toc=toc)
         self._index.append(item)
         return item
@@ -394,8 +407,7 @@ class BaseRfcWriter:
         counter = str(counter)
         autoName = 'Table' + self.nbws_cond + counter
         autoAnchor = 'rfc.table.' + counter
-        item = _RfcItem(autoName, autoAnchor, title=title, anchor=anchor, \
-                       toc=toc)
+        item = _RfcItem(autoName, autoAnchor, counter=counter, title=title, anchor=anchor, toc=toc)
         self._index.append(item)
         return item
 
@@ -406,8 +418,7 @@ class BaseRfcWriter:
         else:
             autoName = '['+counter+']'
         autoAnchor = 'rfc.ref.' + counter
-        item = _RfcItem(autoName, autoAnchor, title=title, anchor=anchor, \
-                       toc=toc)
+        item = _RfcItem(autoName, autoAnchor, counter=counter, title=title, anchor=anchor, toc=toc)
         self._index.append(item)
         return item
 
@@ -415,7 +426,7 @@ class BaseRfcWriter:
         counter = str(counter)
         autoName = 'Comment' + self.nbws_cond + anchor
         autoAnchor = 'rfc.comment.' + counter
-        item = _RfcItem(autoName, autoAnchor, anchor=anchor, toc=False)
+        item = _RfcItem(autoName, autoAnchor, counter=counter, anchor=anchor, toc=False)
         self._index.append(item)
         return item
 
@@ -569,61 +580,66 @@ class BaseRfcWriter:
         """ Returns a lines of lines for the top left header """
         lines = []
         # Document stream / workgroup
-        if self.draft:
-            workgroup = self.r.find('front/workgroup')
-            if workgroup is not None and workgroup.text:
-                lines.append(workgroup.text)
-            else:
-                lines.append(self.boilerplate['draft_workgroup'])
-        else:
-            # Determine 'workgroup' from submissionType
-            subtype = self.r.attrib.get('submissionType', 
-                                         self.defaults['submissionType'])
-            docstream = self.boilerplate['document_stream'].get(subtype)
-            lines.append(docstream)
-
-        # RFC number
-        if not self.draft:
-            lines.append('Request for Comments: ' + self.rfcnumber)
-        else:
-            lines.append('Internet-Draft')
-
-        # Series number
-        category = self.r.attrib.get('category', '')
-        seriesNo = self.r.attrib.get('seriesNo')
-        if seriesNo is not None and category in self.boilerplate['series_name']:
-            lines.append('%s: %s' % (self.boilerplate['series_name'][category], 
-                                     seriesNo))
-
-        # RFC relation notice
-        approved_text = self.draft and '(if approved)' or ''
-        obsoletes = self.r.attrib.get('obsoletes')
-        if obsoletes:
-            lines.append('Obsoletes: %s %s' % (obsoletes, approved_text))
-        updates = self.r.attrib.get('updates')
-        if updates:
-            lines.append('Updates: %s %s' % (updates, approved_text))
-
-        # Cateogory
-        if category:
-            cat_text = self.boilerplate[category]
+        if not self.pis['private']:
             if self.draft:
-                lines.append('Intended status: ' + cat_text)
+                workgroup = self.r.find('front/workgroup')
+                if workgroup is not None and workgroup.text:
+                    lines.append(workgroup.text)
+                else:
+                    lines.append(self.boilerplate['draft_workgroup'])
             else:
-                lines.append('Category: ' + cat_text)
-        else:
-            xml2rfc.log.warn('No category specified for document.')
+                # Determine 'workgroup' from submissionType
+                subtype = self.r.attrib.get('submissionType', 
+                                            self.defaults['submissionType'])
+                docstream = self.boilerplate['document_stream'].get(subtype)
+                lines.append(docstream)
 
-        # Expiration notice for drafts
-        if self.expire_string:
-            lines.append('Expires: ' + self.expire_string)
+            # RFC number
+            if not self.draft:
+                lines.append('Request for Comments: ' + self.rfcnumber)
+            elif not self.pis['private']:
+                lines.append('Internet-Draft')
 
-        # ISSN identifier
-        if not self.draft:
-            lines.append('ISSN: %s' % self.boilerplate['issn'])
+            # Series number
+            category = self.r.attrib.get('category', '')
+            seriesNo = self.r.attrib.get('seriesNo')
+            if seriesNo is not None and category in self.boilerplate['series_name']:
+                lines.append('%s: %s' % (self.boilerplate['series_name'][category], 
+                                         seriesNo))
+
+            # RFC relation notice
+            approved_text = self.draft and '(if approved)' or ''
+            obsoletes = self.r.attrib.get('obsoletes')
+            if obsoletes:
+                wrapper = textwrap.TextWrapper(width=40, subsequent_indent=' '*len('Obsoletes: '))
+                line = 'Obsoletes: %s %s' % (obsoletes, approved_text)
+                lines += wrapper.wrap(line)
+            updates = self.r.attrib.get('updates')
+            if updates:
+                wrapper = textwrap.TextWrapper(width=40, subsequent_indent=' '*len('Updates: '))
+                line = 'Updates: %s %s' % (updates, approved_text)
+                lines += wrapper.wrap(line)
+
+            # Category
+            if category:
+                cat_text = self.boilerplate[category]
+                if self.draft:
+                    lines.append('Intended status: ' + cat_text)
+                else:
+                    lines.append('Category: ' + cat_text)
+            else:
+                xml2rfc.log.warn('No category specified for document.')
+
+            # Expiration notice for drafts
+            if self.expire_string and not self.pis['private']:
+                lines.append('Expires: ' + self.expire_string)
+
+            # ISSN identifier
+            if not self.draft:
+                lines.append('ISSN: %s' % self.boilerplate['issn'])
 
         # Strip any whitespace from XML to make header as neat as possible
-        lines = [ l.strip() for l in lines ]
+        lines = [ l.rstrip() for l in lines ]
         return lines
 
     def _prepare_top_right(self):
@@ -698,6 +714,11 @@ class BaseRfcWriter:
         if preamble is not None:
             self.write_t_rec(preamble, align=figure_align)
 
+        # iref
+        for element in figure:
+            if element.tag == 'iref':
+                self._add_iref_to_index(element)
+
         # Write figure with optional delimiter
         delimiter = self.pis['artworkdelimiter']
         artwork = figure.find('artwork')
@@ -819,6 +840,8 @@ class BaseRfcWriter:
                 self.write_figure(element)
             elif element.tag == 'texttable':
                 self.write_table(element)
+            elif element.tag == 'iref':
+                self._add_iref_to_index(element)
 
         s_count = 1  # Section counter
         
@@ -1067,14 +1090,15 @@ class BaseRfcWriter:
             for t in note.findall('t'):
                 self.write_t_rec(t)
 
-        # Verify that 'ipr' attribute is valid before continuing
-        self._validate_ipr()
+        if not self.pis['private']:
+            # Verify that 'ipr' attribute is valid before continuing
+            self._validate_ipr()
 
-        # "Status of this Memo" section
-        self.write_status_section()
+            # "Status of this Memo" section
+            self.write_status_section()
 
-        # Copyright section
-        self.write_copyright()
+            # Copyright section
+            self.write_copyright()
 
         # Insert the table of contents marker at this position
         toc_enabled = self.pis['toc']
@@ -1107,6 +1131,12 @@ class BaseRfcWriter:
                                    autoAnchor=autoAnchor, level=2)
                 self.write_reference_list(reference_list)
         elif len(references) == 1:
+            if self.eref_list:
+                refs_newcounter = refs_counter + '.1'
+                refs_title = references[0].attrib.get('title', self.pis['refparent'])
+                autoAnchor = 'rfc.references.1'
+                self.write_heading(refs_title, bullet=refs_newcounter + '.',\
+                                   autoAnchor=autoAnchor, level=2)
             self.write_reference_list(references[0])
 
         if self.eref_list:
