@@ -43,7 +43,6 @@ class RawTextRfcWriter(BaseRfcWriter):
         self.iref_marks = {}
 
         # Text lookups
-        self.list_symbols = self.pis['text-list-symbols']
         self.inline_tags = ['xref', 'eref', 'iref', 'cref', 'spanx']
         
         # Custom textwrapper object
@@ -70,7 +69,8 @@ class RawTextRfcWriter(BaseRfcWriter):
     def write_text(self, string, indent=0, sub_indent=0, bullet='',
                   align='left', leading_blankline=False, buf=None,
                   strip=True, edit=False, wrap_urls=True,
-                  fix_sentence_endings=True, source_line=None):
+                  fix_sentence_endings=True, source_line=None,
+                  fix_doublespace=True):
         """ Writes a line or multiple lines of text to the buffer.
 
             Several parameters are included here.  All of the API calls
@@ -112,10 +112,13 @@ class RawTextRfcWriter(BaseRfcWriter):
                 string = string.lstrip()
             if wrap_urls:
                 par = self.wrapper.wrap(string, initial_indent=initial, subsequent_indent=subsequent,
-                                        fix_sentence_endings=fix_sentence_endings)
+                                        fix_sentence_endings=fix_sentence_endings,
+                                        fix_doublespace=fix_doublespace)
             else:
                 par = self.wrapper.wrap(xml2rfc.utils.urlkeep(string), initial_indent=initial,
-                                        subsequent_indent=subsequent, fix_sentence_endings=fix_sentence_endings)
+                                        subsequent_indent=subsequent,
+                                        fix_sentence_endings=fix_sentence_endings,
+                                        fix_doublespace=fix_doublespace)
             if len(par) == 0 and bullet:
                 par = [ initial ]                    
             if align == 'left':
@@ -191,7 +194,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 if t_count > 0 and self.pis['subcompact'] == 'yes':
                     leading_blankline = False
                 if style == 'symbols':
-                    bullet = self.list_symbols[level % len(self.list_symbols)]
+                    bullet = self.pis['text-list-symbols'][level % len(self.pis['text-list-symbols'])]
                     bullet += '  '
                 elif style == 'numbers':
                     bullet = self._format_counter("%d.", t_count+1, listlength)
@@ -285,7 +288,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 indent = 3
                 sub_indent = indent + len(bullet)
                 pagestr = '%4s' % item.page
-                lines = textwrap.wrap(bullet + item.title, 
+                lines = textwrap.wrap(bullet + (item.title if item.title else ""),
                                       self.width - len(pagestr),
                                       initial_indent=' ' * indent,
                                       subsequent_indent=' ' * sub_indent)
@@ -305,6 +308,26 @@ class RawTextRfcWriter(BaseRfcWriter):
 
     def write_iref_index(self):
         """ Write iref index to a temporary buffer and return """
+        def pagelist(pages):
+            pages = list(set(pages))
+            pages.sort()
+            # find ranges
+            items = []
+            prev = None
+            for page in pages:
+                if prev and (page - prev == 1):
+                    items[-1].append(page)
+                else:
+                    items.append([page])
+                prev = page
+            pagelist = []
+            for item in items:
+                if len(item) == 1:
+                    pagelist.append("%d"%item[0])
+                else:
+                    pagelist.append("%d-%d"%(item[0], item[-1]))
+            return ", ".join(pagelist)
+
         if self.iref_marker < 1:
             # iref is either disabled, or the pointer was messed up
             return ['']
@@ -323,13 +346,13 @@ class RawTextRfcWriter(BaseRfcWriter):
             for item in sorted(alpha_bucket[letter]):
                 pages = self._iref_index[item].pages
                 # Write item
-                self.write_text(item + '  ' + ', '.join(map(str, pages))
-                                                        , indent=6, buf=tmpbuf)
+                self.write_text(item + '  ' + pagelist(pages)
+                                                        , indent=6, buf=tmpbuf, fix_doublespace=False)
                 for subitem in sorted(self._iref_index[item].subitems.keys()):
-                    pages = self._iref_index[item].subitems[subitem].pages
+                    pages = set(self._iref_index[item].subitems[subitem].pages)
                     # Write subitem
-                    self.write_text(subitem + '  ' + ', '.join(map(str,pages))
-                                                        , indent=9, buf=tmpbuf)
+                    self.write_text(subitem + '  ' + pagelist(pages)
+                                                        , indent=9, buf=tmpbuf, fix_doublespace=False)
         return tmpbuf
 
     def _expand_xref(self, xref):
@@ -413,16 +436,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                     if self.indexmode:
                         self.eref_list.append([self.eref_count, element])
             elif element.tag == 'iref':
-                item = element.attrib.get('item', None)
-                if item:
-                    subitem = element.attrib.get('subitem', None)
-                    self._make_iref(item, subitem)
-                    # Store the buffer position for pagination data later
-                    pos = len(self.buf)
-                    if not self.indexmode:
-                        if pos not in self.iref_marks:
-                            self.iref_marks[pos] = []
-                        self.iref_marks[pos].append((item, subitem))
+                self._add_iref_to_index(element)
             elif element.tag == 'cref' and \
                 self.pis['comments'] == 'yes':                
                 # Render if processing instruction is enabled
@@ -435,7 +449,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 if self.pis['inline'] == 'yes':
                     if anchor:
                         anchor = anchor + ': '
-                    source = element.attrib.get('source', None)
+                    source = element.attrib.get('source', "")
                     if source:
                         source = " --" + source
                     if element.text:
@@ -509,7 +523,8 @@ class RawTextRfcWriter(BaseRfcWriter):
             longest_line = max(len(line.rstrip()) for line in lines)
             if (longest_line > self.width-indent):
                 new_indent = max(self.width - longest_line, 0)
-                xml2rfc.log.warn('artwork outdented %s characters to avoid overruning right margin around input line %s)' % (indent - new_indent, source_line if source_line else 0))
+                if not self.indexmode:
+                    xml2rfc.log.warn('artwork outdented %s characters to avoid overrunning right margin around input line %s)' % (indent - new_indent, source_line if source_line else 0))
                 indent = new_indent
 
             # Trim first and last lines if they are blank, whitespace is handled
@@ -743,30 +758,31 @@ class RawTextRfcWriter(BaseRfcWriter):
             annotation = ref.find('annotation')
             # Use anchor or num depending on PI
             if self.pis['symrefs'] == 'yes':
-                bullet = '[' + ref.attrib.get('anchor', str(i + self.ref_start)) + ']'
+                key = ref.attrib.get('anchor', str(i + self.ref_start))
             else:
-                bullet = '[' + str(i + self.ref_start) + ']'
-            refdict[bullet] = ''.join(refstring)
-            refsource[bullet] = ref.sourceline
-            refkeys.append(bullet)
+                key = str(i + self.ref_start)
+            refdict[key] = ''.join(refstring)
+            refsource[key] = ref.sourceline
+            refkeys.append(key)
             # Add annotation if it exists to a separate dict
             if annotation is not None and annotation.text:
                 # Render annotation as a separate paragraph
                 remainder = annotation.getchildren()
-                annotationdict[bullet] = annotation.text
+                annotationdict[key] = annotation.text
                 if len(remainder) > 0:
                     inline_txt, remainder = self._combine_inline_elements(remainder)
-                    annotationdict[bullet] += inline_txt
+                    annotationdict[key] += inline_txt
         self.ref_start += i + 1
         # Don't sort if we're doing numbered refs; they are already in
         # numeric order, and if we sort, they will be sorted alphabetically,
         # rather than numerically ... ( i.e., [10], [11], ... [19], [1], ... )
         if self.pis['sortrefs'] == 'yes' and self.pis['symrefs'] == 'yes' :
-            refkeys = sorted(refkeys)
+            refkeys.sort()
         # Hard coded indentation amount
         refindent = 11
         for key in refkeys:
-            self.write_ref_element(key, refdict[key], refindent, source_line=refsource[key])
+            reflabel = '['+key+']'
+            self.write_ref_element(reflabel, refdict[key], refindent, source_line=refsource[key])
             # Render annotation as a separate paragraph
             if key in annotationdict:
                 self.write_text(annotationdict[key], indent=refindent + 3,
